# Redis configuration file
# redis-server /path/to/redis.conf

# for basic
port        {{port}}
daemonize   {{daemonize}}
bind        {{bind}}
dir         {{redis_dir}}


unixsocket  {{redis_dir}}/redis.sock
pidfile     {{redis_dir}}/redis.pid
logfile     {{redis_dir}}/redis.log
dbfilename  {{dbfilename}}
loglevel    {{loglevel}}

protected-mode  {{protected_mode}}
tcp-backlog     {{tcp_backlog}}
unixsocketperm  700
tcp-keepalive   300

{% if is_replica %}
# for replica
replicaof {{replicaof}}
{% endif %}


# -- ~ _ ~    ~ _ ~     ~ _ ~ -- 
# base on redis-7.0.11
# generated by https://www.sqlpy.com at {{now}}
# wechat: jianglegege
# email: 1721900707@qq.com
# -- ~ _ ~ --
