# -*- coding: utf-8 -*-


""" dbm-agent 版本号管理
"""


DBM_AGENT_MAJOR_VERSION = 8
DBM_AGENT_MINOR_VERSION = 33
DBM_AGENT_PATCH_VERSION = 13


# dbm-agent 版本号
DBM_AGENT_VESION = (
    f"{DBM_AGENT_MAJOR_VERSION}.{DBM_AGENT_MINOR_VERSION}.{DBM_AGENT_PATCH_VERSION}"
)
DBM_VERSION_NUMBER = DBM_AGENT_VESION
VERSION = DBM_AGENT_VESION
